/*
 *  Copyright (c) 2014 Stefan Taferner <stefan.taferner@gmx.at>
 *                2015 Deti Fliegl <deti@fliegl.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 */

#ifndef app_4sense_h
#define app_4sense_h

#include <sblib/types.h>

#define MAX_CHANNELS	4

// EEPROM addresses 4Sense VD
#define RE_GW_S_FAKT    0x1B1    // 0
#define RE_SEND_GW      0x1B2    // 0
#define RE_MW_S_FAKT    0x1B3    // *n

#define SENSOR_TYPE     0x1B7    // *n

#define TEMP_OFFSET     0x1BB    // *2n
#define HUMID_OFFSET    0x1BC    // *2n

#define TEMP_MESSDIFF   0x1C3    // *2n Bit6..0: difference, Bit7: enable
#define HUMID_MESSDIFF  0x1C4    // *2n

#define TEMP_ZYKLSEND   0x1CB    // *2n
#define HUMID_ZYKLSEND  0x1CC    // *2n
#define THBASE_ZYKLSEND 0x1D3    // *1/2n

#define TH_TYPE         0x1D7    // *n  Bit 0,2,4,6: 0=, 1=
#define GW_REAKTION     0x1DB    // *1/2n
#define GW_ZYKL_BASE    0x1DC    // 2*n
#define GW_ZYKL_FAKT    0x1E3    // n
#define GW_SCHWELLE1L   0x1E7    // 3*n
#define GW_SCHWELLE1H   0x1E8    // 3*n
#define GW_SCHWELLE2L   0x1E9    // 3*n
#define GW_SCHWELLE3L   0x1EB    // 3*n


// Com objects

// com-object for channel 0. The com-objects 1-7 are for channels 1-7
#define COMOBJ_VALUE0            0

// com-object for threshold 0. channel 0: 0..2, channel1: 3..5, The com-objects 9-19 are for thresholds 1-11
#define COMOBJ_THRESHOLD0        8

// com-object for debug - unused
#define COMOBJ_DEBUG            20


/**
 * A communication object was updated from somebody via the EIB bus.
 *
 * @param objno - the ID of the communication object.
 */
void objectUpdated(int objno);

/**
 * Handle the periodic functions.
 */
void checkPeriodic(void);

/**
 * Initialization of the application
 */
void initApplication(void);

#endif /*app_4sense_h*/
