/*
 *  channel.h - Base class for the different channel operations
 *
 *  Copyright (c) 2015 Martin Glueck <martin@mangari.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 */

#ifndef CHANNEL_H_
#define CHANNEL_H_

#include <sblib/core.h>
//#include <sblib/eib/sblib_default_objects.h>    /* Manuel */

//#include <sblib/types.h>
#include <sblib/timeout.h>
#include "sensor.h"

#define TYPE_TEMP  0
#define TYPE_HUM   1
#define TYPE_VOC   2


/* Base class which defines the interface used by the different channel
 * types for the real implementation.
 *
 * Common code also resides in this class. */
class Channel
{
public:
    Channel(unsigned int type, unsigned int no, Sensor *sensor, unsigned int ch) :
            stype(type), id(no), sensor(sensor), sid(ch)
    {
    };
    virtual void init(void);
    virtual void read(void);
    virtual void checkPeriodic(void);

protected:
    unsigned int stype;    // Sensor type
    unsigned int id;       // channel id
    Sensor *sensor;        // pointer to sensor object
    unsigned int sid;      // sensor id
    unsigned int numval;   // number of measurement values

    unsigned int messwerte[2];  // value of current measurement
    unsigned int lasttemp[2];   // last measurement for comparison
    unsigned int lastsend[2];
    unsigned int timerbase[2];
    unsigned int timercnt[2];

    unsigned int timerbasegw;
    unsigned int timercntgw;
    bool gw_init_done;          // thresholds have been initialized
};

#endif /* CHANNEL_H_ */
