/*
 *  Copyright (c) 2015 Martin Glueck <martin@mangari.org>
 *  Copyright (c) 2015 Deti Fliegl <deti@fliegl.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 */

#include "sensor.h"

//#include <sblib/eib/user_memory.h>
#include <sblib/digital_pin.h>
#include <sblib/io_pin_names.h>
//#include <sblib/timeout.h>
#include <sblib/math.h>           // round()

// SensorTemp Defines
#define SENSORTEMP_PORTPIN         PIO1_7 // GPIO PIN, where the 1-wire data line is connected
#define SENSORTEMP_PARASITE_POWER  true   // Enable the Parasite power (check ds18x20 Datasheet )

// SensorHum Defines
#define SENSORHUM_PORTPIN_1          PIO2_2 // GPIO PIN, where the data line is connected
#define SENSORHUM_PORTPIN_2          PIO2_2 // GPIO PIN, where the data line is connected
#define SENSORHUM_PORTPIN_3          PIO2_2 // GPIO PIN, where the data line is connected
#define SENSORHUM_PORTPIN_4          PIO2_2 // GPIO PIN, where the data line is connected


bool Sensor::read(void)
{
	return false;
}

signed int Sensor::getValue(unsigned int channel)
{
	return 0;
}

unsigned int Sensor::getNum(void)
{
	return noOfChannels;
}


void SensorTemp::Setup(void)
{
  bInitSearch= false;
  ds.DS18x20Init(SENSORTEMP_PORTPIN, SENSORTEMP_PARASITE_POWER);    // Initialize DS18x20 (1-Wire Protocol will be initialized automatically)
  bInitSearch= ds.Search() && ds.m_foundDevices > 0; // Scan the 1-Wire bus for DS18x devices
  noOfChannels = ds.m_foundDevices;
  if (noOfChannels > MAX_CHANNELS)
  {
     noOfChannels = MAX_CHANNELS;
  }
}

bool SensorTemp::read(void)
{
	for (uint8_t ch=0; ch<noOfChannels; ch++)
	{
	    ds.readTemperature(&ds.m_dsDev[ch]);
	}
	return true;
}

signed int SensorTemp::getValue(unsigned int channel)
{
	if (channel < noOfChannels)
	{
        return round(ds.m_dsDev[channel].current_temperature * 10);	// float to 10*int
	}
	else
	{
	    return 0;
	}
}

void SensorHum::Setup(unsigned int id)
{
	switch (id) {
	case 0: dht.DHTInit(SENSORHUM_PORTPIN_1, DHT22);       // Use the DHT22 sensor on PIN
	    break;
	case 1: dht.DHTInit(SENSORHUM_PORTPIN_2, DHT22);       // Use the DHT22 sensor on PIN
	    break;
	case 2: dht.DHTInit(SENSORHUM_PORTPIN_3, DHT22);       // Use the DHT22 sensor on PIN
	    break;
	case 3: dht.DHTInit(SENSORHUM_PORTPIN_4, DHT22);       // Use the DHT22 sensor on PIN
	    break;
	}
    noOfChannels = 2;                            // temp and hum
}

bool SensorHum::read(void)
{
	return (dht.readData());
		    // dht._lastTemperature;
		    // dht._lastHumidity;
		    // float fDewPoint= dht.CalcdewPointFast( dht._lastTemperature, dht._lastHumidity);
}

signed int SensorHum::getValue(unsigned int channel)
{
	switch (channel) {
	case 0:
		return round(dht._lastHumidity*10);	      // float to 10*int
		break;
	case 1:
		return round(dht._lastTemperature*10);    // float to 10*int
		break;
	default:
	    return 0;
		break;
	}
}
